/*
 * HammockMakerDialog.java
 *  
 * Copyright 2009 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.hammingweight.hammock.eclipse.hammockmaker.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class HammockMakerDialog extends Dialog {

	private String classToMock;

	private Text directoryPathText;

	private Text packageNameText;

	private Button cldcButton;
	
	private String directory;

	private String packageName;

	private boolean isCldc11;

	private void selectDirectory() {
		DirectoryDialog dialog = new DirectoryDialog(this.getShell());
		String result = dialog.open();
		if (result != null) {
			this.directoryPathText.setText(result);
		}
	}

	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		container.setLayout(gridLayout);

		// Add labels identifying the class to mock.
		Label label = new Label(container, SWT.NONE);
		GridData gridData = new GridData();
		gridData.horizontalSpan = 3;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = false;
		label.setLayoutData(gridData);
		label.setText("Class to mock:");
		label = new Label(container, SWT.NONE);
		gridData = new GridData();
		gridData.horizontalSpan = 3;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		label.setLayoutData(gridData);
		label.setText(this.classToMock);

		// Must the mock be built for CLDC 1.0 or 1.1?
		this.cldcButton = new Button(container, SWT.CHECK);
		gridData = new GridData();
		gridData.verticalIndent = 15;
		gridData.horizontalSpan = 1;
		gridData.horizontalAlignment = GridData.BEGINNING;
		gridData.grabExcessHorizontalSpace = false;
		this.cldcButton.setLayoutData(gridData);
		this.cldcButton.setSelection(this.isCldc11);
		label = new Label(container, SWT.NONE);
		gridData = new GridData();
		gridData.verticalIndent = 15;
		gridData.horizontalSpan = 1;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		label.setLayoutData(gridData);
		label.setText("Target CLDC 1.1");
		
		// The user must specify a package name.
		label = new Label(container, SWT.NONE);
		gridData = new GridData();
		gridData.verticalIndent = 15;
		gridData.horizontalSpan = 3;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		label.setLayoutData(gridData);
		String mockClass = this.classToMock.substring(this.classToMock
				.lastIndexOf('.') + 1);
		label.setText("Specify the package for the \"Mock" + mockClass
				+ "\" class:");
		this.packageNameText = new Text(container, SWT.BORDER);
		gridData = new GridData();
		gridData.horizontalSpan = 3;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		this.packageNameText.setText(this.packageName);
		this.packageNameText.setLayoutData(gridData);

		// The user must choose a directory.
		label = new Label(container, SWT.NONE);
		gridData = new GridData();
		gridData.verticalIndent = 15;
		gridData.horizontalSpan = 3;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		label.setLayoutData(gridData);
		label
				.setText("Select the directory for the generated source code:");
		this.directoryPathText = new Text(container, SWT.BORDER);
		gridData = new GridData();
		gridData.horizontalSpan = 2;
		gridData.horizontalAlignment = GridData.FILL;
		gridData.grabExcessHorizontalSpace = true;
		this.directoryPathText.setText(this.directory);
		this.directoryPathText.setLayoutData(gridData);
		Button dirButton = new Button(container, SWT.NONE);
		dirButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				selectDirectory();
			}
		});
		dirButton.setText("Browse...");

		return container;
	}

	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText("HammockMaker");
	}

	protected void okPressed() {
		this.directory = this.directoryPathText.getText();
		this.packageName = this.packageNameText.getText();
		this.isCldc11 = this.cldcButton.getSelection();
		super.okPressed();
	}

	public HammockMakerDialog(Shell parentShell, String classToMock,
			String packageName, String directory, boolean isCldc11) {
		super(parentShell);
		this.classToMock = classToMock;
		this.packageName = packageName;
		this.directory = directory;
		this.isCldc11 = isCldc11;
	}

	public String getDirectory() {
		return this.directory;
	}

	public String getPackage() {
		return this.packageName;
	}

	public boolean isCldc11() {
		return isCldc11;
	}

}
