/*
 * HammockMaker.java
 *  
 * Copyright 2008 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.hammingweight.hammockmaker;

import java.util.Vector;

import org.apache.tools.ant.AntClassLoader;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

public class HammockMaker extends Task {

	private Vector<Mock> clazzesToMock = new Vector<Mock>();

	private String mockPackage = "";

	private boolean useCldc11 = true;

	private String dir = ".";

	private Path classpath;

	private boolean failonerror = true;
	
	public void setPackage(String pkg) {
		this.mockPackage = pkg;
	}

	public void setUsecldc11(boolean flag) {
		this.useCldc11 = flag;
	}

	public void setDir(Path path) {
		this.dir = path.toString();
	}

	public void setClasspath(Path classpath) {
		this.classpath = classpath;
	}

	public void setFailonerror(boolean failonerror) {
		this.failonerror = failonerror;
	}
	
	public void addClasspath(Path classpath) {
		setClasspath(classpath);
	}

	public void setRefid(Reference refid) {
		Path classpath = new Path(getProject());
		classpath.setRefid(refid);
		this.setClasspath(classpath);
	}
	
	public void execute() {
		AntClassLoader classLoader = null;

		try {
			for (int i = 0; i < this.clazzesToMock.size(); i++) {
				if (this.classpath != null) {
					Project p = getProject();
					classLoader = p.createClassLoader(this.classpath);
				}

				String qualifiedClassName = this.clazzesToMock.elementAt(i)
						.getMockClassName();
				Class<?> clazzToMock = null;
				if (classLoader != null) {
					try {
						clazzToMock = classLoader.findClass(qualifiedClassName);
					} catch (Exception e) {
						clazzToMock = (Class<?>) Class
								.forName(qualifiedClassName);
					}
				} else {
					clazzToMock = (Class<?>) Class.forName(qualifiedClassName);
				}
				String mockName = clazzToMock.getName();
				mockName = "Mock"
						+ mockName.substring(mockName.lastIndexOf('.') + 1);
				Maker.createMock(clazzToMock, this.mockPackage, mockName,
						this.useCldc11, this.dir);
				if (classLoader != null) {
					classLoader.cleanup();
					classLoader = null;
				}
			}
		} catch (Exception e) {
			if (this.failonerror) {
				throw new BuildException(e.toString());
			} else {
				log(e.toString());
			}
		}
	}

	public void addMock(Mock mock) {
		this.clazzesToMock.add(mock);
	}
}
