/*
 * MockMethodTest.java
 *
 * Created on 27 September 2007, 10:19
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package com.hammingweight.hammock;

import java.io.IOException;

import junit.framework.TestCase;

/**
 * 
 * @author Carl
 */
public class MockMethodTest extends TestCase {

	// Test that a null value is allowed for the return type. That denotes a
	// void method.
	public void testConstructorForVoidMethod() {
		new MockMethod(IMockObject.class, "", new Class[] { String.class },
				new Class[] {}, null, false);
	}

	// Test that a non-null value is allowed for the return type.
	public void testConstructorOkNonVoidMethod() {
		new MockMethod(IMockObject.class, "", new Class[] {},
				new Class[] { NullPointerException.class }, Integer.class,
				false);
	}

	public void testConstructorOk() {
		new MockMethod("foo", 2, IClassDefinitions.OBJECT_CLASS, true);
		new MockMethod("foo", 2, IClassDefinitions.OBJECT_CLASS, true);
	}

	// Test that not supplying a method class throws a NullPointerException.
	public void testConstructorNotOkNullMethodClass() {
		try {
			new MockMethod(null, "", new Class[] {}, new Class[] {},
					Integer.class, false);
			fail();
		} catch (NullPointerException npe) {
		}
	}

	// Test that not supplying a method ID throws a NullPointerException.
	public void testConstructorNotOkNullMethodId() {
		try {
			new MockMethod(String.class, null, new Class[] {}, new Class[] {},
					Integer.class, false);
			fail();
		} catch (NullPointerException npe) {
		}
	}

	// Test that not supplying method argument classes throws a
	// NullPointerException.
	public void testConstructorNotOkNullArgs() {
		try {
			new MockMethod(String.class, "", null, new Class[] {},
					Integer.class, false);
			fail();
		} catch (NullPointerException npe) {
		}
	}

	// Test that not supplying Throwable classes throws a NullPointerException.
	public void testConstructorNotOkNullExceptions() {
		try {
			new MockMethod(String.class, "", new Class[] {}, null,
					Integer.class, false);
			fail();
		} catch (NullPointerException npe) {
		}
	}

	// Test that supplying a null value in the argument list is an error.
	public void testConstructorNotOkNullArgumentClass() {
		try {
			new MockMethod(String.class, "", new Class[] { null },
					new Class[] {}, Integer.class, false);
			fail();
		} catch (NullPointerException npe) {
		}
	}

	// Test that including a class that isn't a subclass of Throwable is an
	// an error.
	public void testConstructorNotOkInvalidThrowable() {
		try {
			new MockMethod(String.class, "", new Class[] {},
					new Class[] { String.class }, Integer.class, false);
			fail();
		} catch (HammockException he) {
			assertEquals(IHammockExceptionErrors.CLASS_IS_NOT_THROWABLE, he
					.getError());
		}
	}

	// Test that including a null object as an exception is an error.
	public void testConstructorNotOkNullException() {
		try {
			new MockMethod(String.class, "", new Class[] {},
					new Class[] { null }, Integer.class, false);
			fail();
		} catch (NullPointerException npe) {
		}
	}

	// Test that a negative number of arguments is Illegal.
	public void testConstructorNotOkNegativeArgCount() {
		try {
			new MockMethod("", -1, IClassDefinitions.OBJECT_CLASS, false);
			fail();
		} catch (IllegalArgumentException iae) {
		}
	}

	// Test that the method's class can be retrieved.
	public void testGetMethodClass() {
		MockMethod mm = new MockMethod(IMockObject.class, "", new Class[] {},
				new Class[] {}, null, false);
		assertEquals(IMockObject.class, mm.getMethodClass());
	}

	// Test that the identifier can be retrieved.
	public void testGetId1() {
		MockMethod mm = new MockMethod(IMockObject.class, "foo",
				new Class[] {}, new Class[] {}, null, false);
		assertEquals("foo", mm.getId());
	}

	// Test that the identifier can be retrieved for the simple constructor.
	public void testGetId2() {
		MockMethod mm = new MockMethod("foo", 5,
				IClassDefinitions.OBJECT_CLASS, false);
		assertEquals("foo", mm.getId());
	}

	// Tests that the argument classes can be retrieved.
	public void testGetArgumentClasses1() {
		MockMethod mm = new MockMethod(IMockObject.class, "", new Class[] {
				String.class, Integer.class }, new Class[] {}, null, false);
		Class[] argClasses = mm.getArgumentClasses();
		assertEquals(2, argClasses.length);
		assertEquals(String.class, argClasses[0]);
		assertEquals(Integer.class, argClasses[1]);
	}

	// Tests that the argument classes can be retrieved for the simple
	// constructor.
	public void testGetArgumentClasses2() {
		MockMethod mm = new MockMethod("", 5, IClassDefinitions.OBJECT_CLASS,
				false);
		Class[] argClasses = mm.getArgumentClasses();
		assertEquals(5, argClasses.length);
		for (int i = 0; i < 5; i++) {
			assertEquals(Object.class, argClasses[i]);
		}
	}

	// Tests that the Throwable classes of the method can be retrieved.
	public void testGetThrowableClasses1() {
		MockMethod mm = new MockMethod(IMockObject.class, "", new Class[] {},
				new Class[] { ArithmeticException.class, Error.class }, null,
				false);
		Class[] throwableClasses = mm.getThrowableClasses();
		assertEquals(2, throwableClasses.length);
		assertEquals(ArithmeticException.class, throwableClasses[0]);
		assertEquals(Error.class, throwableClasses[1]);
	}

	// Tests that the simple constructor allows any exception to be thrown.
	public void testGetThrowableClasses2() {
		MockMethod mm = new MockMethod("", 5, IClassDefinitions.OBJECT_CLASS,
				false);
		Class[] throwableClasses = mm.getThrowableClasses();
		assertEquals(1, throwableClasses.length);
		assertEquals(Throwable.class, throwableClasses[0]);
	}

	// Tests that we can get the return Class for a method.
	public void testGetReturnClass1() {
		MockMethod mm = new MockMethod(IMockObject.class, "", new Class[] {},
				new Class[] {}, Long.class, false);
		assertEquals(Long.class, mm.getReturnClass());
	}

	// Tests that we can get the return Class for a method for the simple
	// constructor.
	public void testGetReturnClass2() {
		MockMethod mm = new MockMethod("", 5, IClassDefinitions.OBJECT_CLASS,
				false);
		assertEquals(Object.class, mm.getReturnClass());
	}

	public void testIsAbstract() {
		MockMethod mm = new MockMethod("", 5, IClassDefinitions.OBJECT_CLASS,
				false);
		assertFalse(mm.isAbstract());
		mm = new MockMethod("", 5, IClassDefinitions.OBJECT_CLASS, true);
		assertTrue(mm.isAbstract());
	}

	public void testValidateThrowable() {
		MockMethod mm = new MockMethod(IMockObject.class, "FOO",
				new Class[] {}, new Class[] { IOException.class }, null, true);

		mm.validateThrowable(new RuntimeException());
		mm.validateThrowable(new Error());
		mm.validateThrowable(new NullPointerException());
		try {
			mm.validateThrowable(new ClassNotFoundException());
			fail("Can't throw CNFE.");
		} catch (HammockException e) {
			assertEquals(IHammockExceptionErrors.INCORRECT_THROWABLE_CLASS, e
					.getError());
		}
		mm.validateThrowable(new IOException());
	}
	
	public void testGetNumberOfArguments() {
		MockMethod mm = new MockMethod(IMockObject.class, "FOO",
				new Class[] {}, new Class[0], null, true);
		assertEquals(0, mm.getNumberOfArguments());
		mm = new MockMethod(IMockObject.class, "FOO",
				new Class[] {Integer.class, String.class}, new Class[0], null, true);
		assertEquals(2, mm.getNumberOfArguments());
	}
}
